// Copyright (C) 2012-2014 Damir Korencic, Strahil Ristov
//
// This file is part of lzlex v0.1
//
// lzlex is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// lzlex is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with lzlex.  If not, see <http://www.gnu.org/licenses/>.

#ifndef ICOMPRESSOR_H
#define	ICOMPRESSOR_H

#include <string>

#include "util/WordFileReader.h"
#include "dictionary/util/WordList.h"
#include "dictionary/char_trie/Trie.h"
#include "util/etimer.h"

using namespace std;

/** Interface for a family of algorithms (strategy)
 * that compress NodeArray's. */
template <typename TNodeArray>
class ICompressor {
private:
    typedef typename TNodeArray::Symbol TSymbol;
    typedef typename TNodeArray::Index TIndex;

public:

    //TODO pogledaj konstruktore kopije
    //ICompressor(const ICompressor& orig);
    virtual ~ICompressor() {};
    
    virtual void compressArray(TNodeArray& array) = 0;

    virtual TNodeArray* compressWords(string fileName);

    // short algorithm description
    virtual string description() = 0;

private:

};

/** Create NodeArray that stores the set of words in the
 * lexicographically sorted file. */
template <typename TNodeArray>
TNodeArray* ICompressor<TNodeArray>::compressWords(string fileName) {
        startEvent("reading_words");
        WordFileReader<TSymbol> reader(fileName);
        WordList<TSymbol>* words = reader.getWords();
        endEvent("reading_words");
        // construct trie
        startEvent("neto_compress");
        startEvent("building_trie");
        Trie<TNodeArray>* trie = new Trie<TNodeArray>(0);
        for (size_t i = 0; i < words->numberOfWords(); ++i)
            trie->insertWord((*words)[i]);        
        delete words;
        TNodeArray* nodes = &(trie->exportNodeArray());
        delete trie;
        endEvent("building_trie");
        // compress trie        
        startEvent("compress_trie");
        this->compressArray(*nodes);
        endEvent("compress_trie");
        return nodes;
}

#endif	/* ICOMPRESSOR_H */


