// Copyright (C) 2012-2014 Damir Korencic, Strahil Ristov
//
// This file is part of lzlex v0.1
//
// lzlex is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// lzlex is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with lzlex.  If not, see <http://www.gnu.org/licenses/>.

#ifndef COMPACTARRAYNODE_H
#define	COMPACTARRAYNODE_H

template <typename TSymbol, typename TIndex>
class CompactArrayNode {

public:

    CompactArrayNode() {};
    CompactArrayNode(const CompactArrayNode& orig);
    virtual ~CompactArrayNode() {};

    TSymbol getSymbol() const { return symbol; }
    TIndex getSibling() const { return sibling; }
    TIndex getNumWords() const { return numOfWords; }
    bool getEow() const { return eow; }
    bool getCow() const { return cow; }

    bool isPointer() const { return (eow == false) && (cow == false); };

    bool operator<(CompactArrayNode n2) const;
    bool operator==(CompactArrayNode n2) const;
    bool operator!=(CompactArrayNode n2) const;

    template <typename T1, typename T2> friend class CompactArray;

private:

    TSymbol symbol;
    TIndex sibling;
    TIndex numOfWords;
    bool eow;
    bool cow;
    // true if numOfWords field is considered, ie. node is form enumerated array
    bool enumerated;

};

template <typename TSymbol, typename TIndex>
CompactArrayNode<TSymbol, TIndex>::CompactArrayNode(const CompactArrayNode& src) {
    symbol = src.symbol;
    sibling = src.sibling;
    numOfWords = src.numOfWords;
    eow = src.eow;
    cow = src.cow;
}

template <typename TSymbol, typename TIndex>
bool CompactArrayNode<TSymbol, TIndex>::operator <(CompactArrayNode n2) const {
    if (symbol < n2.symbol) return true;
    else if (n2.symbol < symbol) return false;

    if (eow < n2.eow) return true;
    else if (n2.eow < eow) return false;

    if (cow < n2.cow) return true;
    else if (n2.cow < cow) return false;

    //TODO na koju vrijednost postaviti sibling node nema pointera
    if (sibling < n2.sibling) return true;
    else if (n2.sibling < sibling) return false;

    if (enumerated) {
        if (numOfWords < n2.numOfWords) return true;
        if (numOfWords > n2.numOfWords) return false;
    }

    return false;
}

template <typename TSymbol, typename TIndex>
bool CompactArrayNode<TSymbol, TIndex>::operator ==(CompactArrayNode n2) const {
    bool equal;
    
    equal = (symbol == n2.symbol && sibling == n2.sibling &&
             eow == n2.eow && cow == n2.cow);

    if (enumerated) equal = equal && (numOfWords == n2.numOfWords);

    return equal;
}

template <typename TSymbol, typename TIndex>
bool CompactArrayNode<TSymbol, TIndex>::operator !=(CompactArrayNode n2) const {
    return !( (*this) == n2 );
}

#endif	/* COMPACTARRAYNODE_H */


