// Copyright (C) 2012-2014 Damir Korencic, Strahil Ristov
//
// This file is part of lzlex v0.1
//
// lzlex is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// lzlex is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with lzlex.  If not, see <http://www.gnu.org/licenses/>.

#ifndef BITPOINTER_H
#define	BITPOINTER_H

#include <cstddef>

#include "util/constants.h"

/** Describes a location of a bit in an array of bit blocks,
 * indexes are zero based. */
class BitPointer {
public:

    size_t blockIndex;
    // position of a bit inside the block
    int bitIndex;

    /** Create a pointer for an array of bit blocks, each blockSize chars wide,
     * pointing to a bit at bitPosition. */
    BitPointer(int blockSize, size_t bitPosition) {
        bitsPerBlock = blockSize * BITS_PER_CHAR;
        blockIndex = bitPosition / bitsPerBlock;
        bitIndex = bitPosition % bitsPerBlock;
    }

    /** Increment bit pointer by 1. */
    inline void increment() {
        bitIndex++;
        if (bitIndex == bitsPerBlock) {
            bitIndex = 0;
            blockIndex++;
        }
    }

    /** Decrement bit pointer by 1. Decremented zero position remains zero. */
    inline void decrement() {
        if (bitIndex != 0) {
            bitIndex--;
        }
        else {
            if (blockIndex == 0) return;
            else {
                blockIndex--;
                bitIndex = bitsPerBlock-1;
            }
        }

    }

    inline bool operator<(BitPointer bp) {
        if (blockIndex < bp.blockIndex) return true;
        if (blockIndex > bp.blockIndex) return false;
        if (bitIndex < bp.bitIndex) return true;
        return false;
    }

private:
    int bitsPerBlock;
};

#endif	/* BITPOINTER_H */


