// Copyright (C) 2012-2014 Damir Korencic, Strahil Ristov
//
// This file is part of lzlex v0.1
//
// lzlex is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// lzlex is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with lzlex.  If not, see <http://www.gnu.org/licenses/>.

#ifndef SERIALIZATION_H
#define	SERIALIZATION_H

#include <cstddef>

#include "util/constants.h"

size_t numberOfBlocks(size_t numberOfBits, size_t fieldSize);
size_t numberOfValues(int numberOfBits);
int numberOfBits(size_t numberOfValues);

/** Return a value of type T with all bits set to zero. */
template <typename T>
inline T zeroBits() {
    long unsigned int const ONE = 1;
    int numOfBits = sizeof(T) * BITS_PER_CHAR;

    T z;
    for (int i = 0; i < numOfBits; ++i)
        z &= ~(ONE << i);

    return z;
}

/** Get number of blocks, each blockSize chars large, required to
 * hold numberOfBits. */
inline size_t numberOfBlocks(size_t numberOfBits, size_t blockSize) {
    //TODO napisati efikasnije
    size_t nob = numberOfBits / (blockSize * BITS_PER_CHAR);
    if (numberOfBits % (blockSize * BITS_PER_CHAR) != 0) nob++;
    return nob;
}

/** Get number of different values representable by numberOfBits. */
inline size_t numberOfValues(int numberOfBits) {
    if (numberOfBits <= 0) return 0;
    size_t num = 1;
    return num << numberOfBits;
}

/** Calculate number of bits necessary to store numberOfValues different values. */
inline int numberOfBits(size_t numberOfValues) {
    int bits = 1; size_t vals = 2;
    while (vals < numberOfValues) {
        bits++;
        vals *= 2;
    }
    return bits;
}

/** Calculate min. number of bits that can store numbers [0,..,n].  */
inline int minBitsForNumber(size_t n) {
    return numberOfBits(n+1);
}

#endif	/* UTILS_H */


