// Copyright (C) 2012-2014 Damir Korencic, Strahil Ristov
//
// This file is part of lzlex v0.1
//
// lzlex is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// lzlex is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with lzlex.  If not, see <http://www.gnu.org/licenses/>.

#ifndef WORDCOMPARATOR_H
#define	WORDCOMPARATOR_H

#include "node_array/types/symbol.h"
#include "util/utils.h"

/** Generic word comparator, lexicographically compares two strings of symbols. */
template <typename TSymbol>
class WordComparator {
public:
    virtual ~WordComparator() {}

    inline bool operator()(TSymbol const *w1, TSymbol const *w2) const {
        return compare(w1, w2);
    }

    /** Comparison that can be used without object instantiation.
     * Compare two words lexicographicaly, return true if w1 is less than w2. */
    static inline bool compare(TSymbol const *w1, TSymbol const *w2) {
        size_t i;
        for (i = 0; w1[i] != zeroSymbol<TSymbol>(); ++i) {
            if (w2[i] == zeroSymbol<TSymbol>()) return false;
            if (w1[i] < w2[i]) return true;
            if (w1[i] > w2[i]) return false;
        }
        // w2 ends at the same positions, so w1 == w2
        if (w2[i] == zeroSymbol<TSymbol>()) return false;
        // w1 is prefix of w2
        else return true;
    }

private:

};

#endif	/* WORDCOMPARATOR_H */


